﻿using Newtonsoft.Json;
using Oxide.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using uLink;
using UnityEngine;

#pragma warning disable CS0162

namespace Oxide.Plugins
{
    [Info("NoEscape ItemV2", "Kaidoz", "1.2.7")]
    [Description("Blocks system for Hurtworld ItemV2.")]
    public class NoEscapeV2 : HurtworldPlugin
    {
        /* SUPPORT 
         *  Discord: Kaidoz#3059
         *  Steam: https://steamcommunity.com/id/ka1doz/
         * */

        #region Class

        public class OnRaid
        {
            public ulong steamid { get; set; }
            public string time { get; set; }
            public OnRaid(ulong steamid, string time)
            {
                this.steamid = steamid;
                this.time = time;
            }
        }

        #endregion

        #region Data

        List<OnRaid> onraids = new List<OnRaid>();

        List<string> blocked_items = new List<string>()
        {
            "ConstructionHammer",
            "Titranium Workbench",
            "Workbench",
            "Storage Locker",
            "Storage Box",
            "OwnershipStake",
            "Sign",
            "Recycler",
            "Fridge",
            "Torch",
            "FirePit",
            "AutomaticDrillRed",
            "BedBuilder",
            "AutomaticDrillBlue",
            "Campfire",
            "AutomaticDrillGreen",
            "Storage Chest"
        };

        #endregion

        #region Lang

        private void LoadDefaultMessages()
        {
            lang.RegisterMessages(new Dictionary<string, string>
            {
                {"RaidBlock Started", "<color=red><b>[NoEscape]</b></color> Начался рейд! У вас блок {0} минут"},
                {"Message about Timer", "<color=red><b>[NoEscape]</b></color> У вас блок на {0} секунд."}
            }, this, "ru");

            lang.RegisterMessages(new Dictionary<string, string>
            {
                {"RaidBlock Started", "<color=red><b>[NoEscape]</b></color> Raid started! You have a block for {0} minutes."},
                {"Message about Timer", "<color=red><b>[NoEscape]</b></color> You have a block for {0} seconds."}
            }, this);
        }

        #endregion

        #region Config

        private new void LoadConfig()
        {
            base.LoadConfig();
            try
            {
                _config = Config.ReadObject<Configuration>();
                if (_config == null) throw new Exception();
            }
            catch
            {
                Config.WriteObject(_config, false, $"{Interface.Oxide.ConfigDirectory}/{Name}.jsonError");
                PrintError("The configuration file contains an error and has been replaced with a default config.\n" +
                           "The error configuration file was saved in the .jsonError extension");
                LoadDefaultConfig();
            }
            SaveConfig();
        }

        protected override void LoadDefaultConfig()
        {
            PrintWarning("Creating new config file for NoEscapeV2...");
            _config = new Configuration()
            {
                runforadmin = true,
                freezeitem = false,
                exployactiveblock = false,
                logs = true,
                timec4 = 5,
                timedrill = 7,
                radius = 150
            };
            SaveConfig();
        }

        protected override void SaveConfig() => Config.WriteObject(_config);

        Configuration _config;

        public class Configuration
        {
            [JsonProperty("RaidBlock for Admins")]
            public bool runforadmin;

            [JsonProperty("RaidBlock at Explosixe(not enabled)")]
            public bool exployactiveblock;

            [JsonProperty("Logs")]
            public bool logs;

            [JsonProperty("RaidBlock time of C4")]
            public int timec4;

            [JsonProperty("RaidBlock time of RaidDrill")]
            public int timedrill;

            [JsonProperty("Radius of RaidBlock")]
            public int radius;

            [JsonProperty("Freeze item when dropped")]
            public bool freezeitem;
        }

        #endregion

        #region Work Data

        void SaveData() => Interface.Oxide.DataFileSystem.WriteObject("NoEscape/players", onraids);

        void LoadData()
        {
            if (Interface.Oxide.DataFileSystem.ExistsDatafile("NoEscape/players"))
                onraids = Interface.Oxide.DataFileSystem.ReadObject<List<OnRaid>>("NoEscape/players");

            if (onraids.Count() == 0)
                return;
            for (int o = 0; o < onraids.Count(); o++)
            {
                try
                {
                    if (DateTime.Compare(DateTime.Parse(onraids[o].time), DateTime.Now) < 0)
                        onraids.Remove(onraids[o]);
                }
                catch { }
            }
            SaveData();

        }

        #endregion

        #region Hooks

        void Loaded()
        {
            LoadData();
            LoadConfig();
        }

        void OnPlayerInput(PlayerSession session)
        {
            if (session.WorldPlayerEntity != null && israida(session))
                checkitem(session);
        }

        void OnEntitySpawned(HNetworkView data)
        {
            if (data.gameObject.name.Contains("C4"))
            {
                var sessions = GameManager.Instance?.GetSessions()?.Values.Where(IsValidSession).ToList();
                var playersNear =
                  sessions.Where(s =>
                          Vector3.Distance(s.WorldPlayerEntity.transform.position,
                              data.gameObject.transform.position) <= _config.radius)
                      .ToList();
                blockplayers(playersNear, _config.timec4);
            }

            if (data.gameObject.name.Contains("RaidDrill") && !data.gameObject.name.Contains("Item"))
            {
                var sessions = GameManager.Instance?.GetSessions()?.Values.Where(IsValidSession).ToList();
                var playersNear =
                  sessions.Where(s =>
                          Vector3.Distance(s.WorldPlayerEntity.transform.position,
                              data.gameObject.transform.position) <= _config.radius)
                      .ToList();
                blockplayers(playersNear, _config.timedrill);
            }
        }

        #endregion

        #region Methods

        void addraid(PlayerSession session, int time)
        {
            clearexist((ulong)session.SteamId, time);
            DateTime blocktime = DateTime.Now.AddMinutes(time);
            onraids.Add(new OnRaid((ulong)session.SteamId, blocktime.ToString()));
            reply(session, string.Format(Msg("RaidBlock Started", session.SteamId.ToString()), time));
            SaveData();
        }

        void checkitem(PlayerSession session)
        {
            var ehs = session.WorldPlayerEntity.GetComponent<EquippedHandlerServer>();
            var item = ehs.GetEquippedItem();
            if (item == null)
                return;

            if (CheckItem(item.Generator.name))
            {
            A:
                if ((from x in onraids where x.steamid == (ulong)session.SteamId select x).Count() > 0)
                {
                    var dat = (from x in onraids where x.steamid == (ulong)session.SteamId select x).FirstOrDefault();
                    int result = DateTime.Compare(DateTime.Parse(dat.time), DateTime.Now);
                    if (result > 0)
                    {
                        int countraid = Convert.ToInt32(DateTime.Parse(dat.time).Subtract(DateTime.Now).TotalSeconds);
                        reply(session, string.Format(Msg("Message about Timer", session.SteamId.ToString()), countraid));//$"<color=red><b>[NoEscape]</b></color> У вас блок на {countraid} секунд.");
                        if(!ReturnItemInInventory(session, ehs.GetEquippedSlotNumber()))
                            dropItem(session);
                    }
                    else
                    {
                        onraids.Remove(dat);
                        SaveData();
                        goto A;
                    }
                }
            }
        }

        void dropItem(PlayerSession session)
        {
            var ehs = session.WorldPlayerEntity.GetComponent<EquippedHandlerServer>();
            var inv = session.WorldPlayerEntity.GetComponent<PlayerInventory>();
            dropSlot(session, inv, ehs.GetEquippedSlotNumber());
        }

        void dropSlot(PlayerSession session, PlayerInventory inv, int slot)
        {
            var item = inv.GetSlot(slot);
            var IGM = Singleton<GlobalItemManager>.Instance;
            var obj = GlobalItemManager.SpawnWorldItem(item, session.WorldPlayerEntity.transform.position + new Vector3(0, 1f, 0f), Vector3.zero);
            if(_config.freezeitem)
                obj.gameObject.AddComponent<Freeze>();
        }


        bool israida(PlayerSession session)
        {
            if ((from x in onraids where x.steamid == (ulong)session.SteamId select x).Count() > 0)
                return true;
            return false;
        }


        void clearexist(ulong steamid, int time)
        {
            var d = (from x in onraids where x.steamid == steamid select x);
            while (d.Count() > 0)
            {
                int countraid = Convert.ToInt32(DateTime.Parse(d.FirstOrDefault().time).Subtract(DateTime.Now).Minutes);
                if (countraid > time)
                    break;

                onraids.Remove(d.FirstOrDefault());
                d = (from x in onraids where x.steamid == steamid select x);
            }
        }

        void blockplayers(List<PlayerSession> playersNear, int time)
        {
            foreach (PlayerSession session in playersNear)
            {
                if (session.IsAdmin)
                {
                    if (_config.runforadmin)
                        addraid(session, time);
                }
                else
                    addraid(session, time);
            }
            if (_config.logs)
                Logs(playersNear);
        }

        #endregion

        #region Mono

        public class Freeze : UnityEngine.MonoBehaviour
        {
            private void OnCollisionEnter(Collision other)
            {
                var rigidbody = gameObject.GetComponent<Rigidbody>();
                rigidbody.constraints = RigidbodyConstraints.FreezePosition;
            }
        }

        #endregion

        /// <summary>
        /// Check player for raidblock
        /// </summary>
        /// <param name="session"></param>
        /// <returns></returns>
        bool isRaid(PlayerSession session)
        {
        A:
            if (israida(session))
            {
                var _fira = (from x in onraids where x.steamid == (ulong)session.SteamId select x);
                var dat = _fira.FirstOrDefault();
                int countraid = Convert.ToInt32(DateTime.Parse(dat.time).Subtract(DateTime.Now).TotalSeconds);
                if (countraid > 0)
                {
                    reply(session, string.Format(Msg("Message about Timer", session.SteamId.ToString()), countraid));
                    return true;
                }
                else
                {
                    onraids.Remove(dat);
                    SaveData();
                    goto A;
                }
            }

            return false;
        }

        bool ReturnItemInInventory(PlayerSession session, int slot)
        {
            var inv = session.WorldPlayerEntity.GetComponent<PlayerInventory>();
            var Item = inv.GetSlot(slot);
            for(int d = 9;d<inv.Capacity-1;d++)
            {
                if(inv.GetSlot(d)==null)
                {
                    inv.SetSlot(d, Item);
                    inv.RemoveItem(d);
                    return true;
                }
            }
            return false;
        }

        #region Block Kits and TP

        /// <summary>
        /// Block for ExtTeleport
        /// </summary>
        /// <param name="session"></param>
        /// <returns></returns>
        object canExtTeleport(PlayerSession session)
        {
            if (isRaid(session))
            {
                return true;
            }
            return null;
        }


        /// <summary>
        /// Block for Kits
        /// </summary>
        /// <param name="session"></param>
        /// <returns></returns>
        object canRedeemKit(PlayerSession session)
        {
            if (isRaid(session))
            {
                return true;
            }
            return null;
        }
        #endregion

        #region Helpers

        string Msg(string msg, string SteamId = null) => lang.GetMessage(msg, this, SteamId);

        void reply(PlayerSession session, string msg) => hurt.SendChatMessage(session, null, msg);

        /// <summary>
        /// Check exist id from ForbiddenItems
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        bool CheckItem(string name)
        {
            int count = (from x in blocked_items where x == name select x).Count();
            if (count >= 1)
                return true;

            return false;
        }

        ItemGeneratorAsset getItem(int id) => GlobalItemManager.Instance.GetGenerators()[id];

        /// <summary>
        /// Logs
        /// </summary>
        /// <param name="sess"></param>
        void Logs(List<PlayerSession> sess)
        {
            string msg = "[" + DateTime.Now + "]" + "Players are Raid Blocked:\n";
            foreach (var s in sess)
            {
                msg += s.Identity.Name + $" ({s.SteamId})" + "\n";
            }
            LogToFile("NoEscape", msg, this);
        }

        public bool IsValidSession(PlayerSession session)
        {
            return session?.SteamId != null && session.IsLoaded && session.Identity.Name != null && session.Identity != null &&
                   session.WorldPlayerEntity?.transform?.position != null;
        }

        #endregion
    }
}
